<?php

declare(strict_types=1);

namespace Drupal\Component\OTUnit;

use OpenTelemetry\Config\SDK\ComponentProvider\OutputStreamParser;
use OpenTelemetry\Contrib\Otlp\ContentTypes;
use OpenTelemetry\Contrib\Otlp\LogsExporter;
use OpenTelemetry\SDK\Common\Configuration\Configuration;
use OpenTelemetry\SDK\Common\Configuration\Variables;
use OpenTelemetry\SDK\Logs\LogRecordExporterFactoryInterface;
use OpenTelemetry\SDK\Logs\LogRecordExporterInterface;
use OpenTelemetry\SDK\Registry;

/**
 * Constructs an otlp log exporter writing collected data to public files.
 */
class FileLogRecordExporterFactory implements LogRecordExporterFactoryInterface {

  /**
   * {@inheritdoc}
   */
  public function create(): LogRecordExporterInterface {
    return new LogsExporter(Registry::transportFactory('stream')->create(
      endpoint: $this->getEndpoint(),
      contentType: ContentTypes::NDJSON,
    ));
  }

  /**
   * Returns path to otlp file.
   */
  private function getEndpoint(): string {
    $uri = Configuration::has(Variables::OTEL_EXPORTER_OTLP_LOGS_ENDPOINT) ?
      Configuration::getString(Variables::OTEL_EXPORTER_OTLP_LOGS_ENDPOINT) :
      Configuration::getString(Variables::OTEL_EXPORTER_OTLP_ENDPOINT);
    return OutputStreamParser::parse($uri);
  }

}
