<?php

declare(strict_types=1);

namespace Drupal\Component\OTUnit;

use OpenTelemetry\Config\SDK\ComponentProvider\OutputStreamParser;
use OpenTelemetry\Contrib\Otlp\ContentTypes;
use OpenTelemetry\Contrib\Otlp\MetricExporter;
use OpenTelemetry\SDK\Common\Configuration\Configuration;
use OpenTelemetry\SDK\Common\Configuration\Variables;
use OpenTelemetry\SDK\Metrics\Data\Temporality;
use OpenTelemetry\SDK\Registry;
use OpenTelemetry\SDK\Metrics\MetricExporterFactoryInterface;
use OpenTelemetry\SDK\Metrics\MetricExporterInterface;

/**
 * Constructs an otlp metrics exporter writing collected data to public files.
 */
class FileMetricExporterFactory implements MetricExporterFactoryInterface {

  /**
   * {@inheritdoc}
   */
  public function create(): MetricExporterInterface {
    return new MetricExporter(Registry::transportFactory('stream')->create(
      endpoint: $this->getEndpoint(),
      contentType: ContentTypes::NDJSON,
    ), $this->getTemporality());
  }

  /**
   * Returns path to otlp file.
   */
  private function getEndpoint(): string {
    $uri = Configuration::has(Variables::OTEL_EXPORTER_OTLP_METRICS_ENDPOINT) ?
      Configuration::getString(Variables::OTEL_EXPORTER_OTLP_METRICS_ENDPOINT) :
      Configuration::getString(Variables::OTEL_EXPORTER_OTLP_ENDPOINT);
    return OutputStreamParser::parse($uri);
  }

  /**
   * Returns metrics temporality preference.
   */
  private function getTemporality(): string|null {
    $value = Configuration::getEnum(Variables::OTEL_EXPORTER_OTLP_METRICS_TEMPORALITY_PREFERENCE);
    return match (strtolower($value)) {
      'cumulative' => Temporality::CUMULATIVE,
      'delta' => Temporality::DELTA,
      'lowmemory' => NULL,
      default => throw new \UnexpectedValueException('Unknown temporality: ' . $value),
    };
  }

}
