<?php

declare(strict_types=1);

namespace Drupal\Component\OTUnit;

use OpenTelemetry\Config\SDK\ComponentProvider\OutputStreamParser;
use OpenTelemetry\Contrib\Otlp\ContentTypes;
use OpenTelemetry\Contrib\Otlp\SpanExporter;
use OpenTelemetry\SDK\Common\Configuration\Configuration;
use OpenTelemetry\SDK\Common\Configuration\Variables;
use OpenTelemetry\SDK\Registry;
use OpenTelemetry\SDK\Trace\SpanExporter\SpanExporterFactoryInterface;
use OpenTelemetry\SDK\Trace\SpanExporterInterface;

/**
 * Constructs an otlp span exporter writing collected data to a file.
 */
class FileSpanExporterFactory implements SpanExporterFactoryInterface {

  /**
   * {@inheritdoc}
   */
  public function create(): SpanExporterInterface {
    return new SpanExporter(Registry::transportFactory('stream')->create(
      endpoint: $this->getEndpoint(),
      contentType: ContentTypes::NDJSON,
    ));
  }

  /**
   * Returns path to otlp file.
   */
  private function getEndpoint(): string {
    $uri = Configuration::has(Variables::OTEL_EXPORTER_OTLP_TRACES_ENDPOINT) ?
      Configuration::getString(Variables::OTEL_EXPORTER_OTLP_TRACES_ENDPOINT) :
      Configuration::getString(Variables::OTEL_EXPORTER_OTLP_ENDPOINT);
    return OutputStreamParser::parse($uri);
  }

}
